﻿using CalendarNotepad.Extends;
using CalendarNotepad.Models;
using SqlSugar;

namespace CalendarNotepad
{
    public partial class WorkMain:Form
    {
        public DbContext Db { get; set; } = new DbContext();

        public WorkMain() {
            InitializeComponent();
            this.rtbMsg.EnableAutoDragDrop = true;
            this.DataReset();
            this.rtbMsg.TextChanged += RtbMsg_TextChanged;
            foreach(var c in this.C_WorkTypes.Controls) {
                var rab = c as RadioButton;
                if(rab != null) {
                    rab.CheckedChanged += Rab_CheckedChanged;
                }
            }
            foreach(var c in this.C_WorkOut.Controls) {
                var rab = c as RadioButton;
                if(rab != null) {
                    rab.CheckedChanged += Rab_CheckedChanged;
                }
            }
            var fieViewWidth = this.FileListView.Width;
            this.FileListView.Columns.Add(new ColumnHeader {
                Text = "附件",Width = fieViewWidth - 200,TextAlign = HorizontalAlignment.Left,
            });
            this.FileListView.Columns.Add(new ColumnHeader {
                Text = "类型",Width = 100,TextAlign = HorizontalAlignment.Left,
            });
            this.FileListView.Columns.Add(new ColumnHeader {
                Text = "文件指纹",Width = 100,TextAlign = HorizontalAlignment.Left,
            });
        }

        private void Rab_CheckedChanged(object? sender,EventArgs e) {
            this.DataChanged();
        }

        private void RtbMsg_TextChanged(object? sender,EventArgs e) {
            this.DataChanged();
        }

        private void WorkMain_FormClosing(object sender,FormClosingEventArgs e) {
            if(this.Db != null) {
                this.Db.Close();
                this.Db.Dispose();
            }
        }

        private void mcWorkDay_DateChanged(object sender,DateRangeEventArgs e) {
            var wd = this.mcWorkDay.SelectionStart.ToString("yyyyMMdd");
            this.FileListView.Items.Clear();
            var qu = this.Db.Queryable<WorkUnit>().Where(a => a.WorkDay == wd).ToList();
            if(!qu.Any()) {
                this.rtbMsg.Text = "";
                this.SetWorkType("休假");
                this.SetWorkOut("无");
            }
            if(qu.Any()) {
                var fir = qu.First();
                this.rtbMsg.Text = fir.WorkMessage;
                this.SetWorkType(fir.WorkType);
                this.SetWorkOut(fir.Workout ?? "无");

                var plsq = this.Db.Queryable<WorkUnitPlusFile>().Where(a => a.WorkDay == wd).ToList();
                if(plsq.Any()) {
                    foreach(var i in plsq) {
                        var lit1 = new ListViewItem(i.FileName);
                        lit1.SubItems.Add(new ListViewItem.ListViewSubItem { Text = "已载入" });
                        lit1.SubItems.Add(new ListViewItem.ListViewSubItem { Text = i.FileKey });
                        this.FileListView.Items.Add(lit1);
                    }
                }
            }
            this.DataReset();
            Task.Factory.StartNew(() => {
                this.Invoke(() => {
                    this.mcWorkDay.BoldedDates = GetBoldDays().ToArray();
                });
            });
        }

        private List<DateTime> GetBoldDays() {
            var year = this.mcWorkDay.SelectionStart.Year;
            var month = this.mcWorkDay.SelectionStart.Month;
            var days = new List<DateTime>();
            for(int i = 1;i < 32;i++) {
                var wd = $"{year}{month.ToString("D2")}{i.ToString("D2")}";
                var qu = this.Db.Queryable<WorkUnit>().Where(a => a.WorkDay == wd && a.WorkMessage != "").ToList();
                if(qu.Any()) {
                    days.Add(new DateTime(year,month,i));
                }
            }

            return days;
        }

        private void WorkMain_Load(object sender,EventArgs e) {
            mcWorkDay_DateChanged(sender,new DateRangeEventArgs(this.mcWorkDay.SelectionStart,this.mcWorkDay.SelectionEnd));
        }

        /// <summary>
        /// 获取工作类型
        /// </summary>
        /// <returns></returns>
        private string GetWorkType() {
            foreach(Control c in this.C_WorkTypes.Controls) {
                var rb = c as RadioButton;
                if(rb != null && rb.Checked) {
                    return rb.Text;
                }
            }
            return "";
        }
        /// <summary>
        /// 设置工作类型
        /// </summary>
        /// <param name="t">工作类型</param>
        private void SetWorkType(string? t) {
            if(t.IsNullOrEmpty()) {
                return;
            }
            foreach(Control c in this.C_WorkTypes.Controls) {
                var rb = c as RadioButton;
                if(rb != null && rb.Text == t) {
                    rb.Checked = true;
                    return;
                }
            }
        }

        private void btSave_Click(object sender,EventArgs e) {
            var wd = this.mcWorkDay.SelectionStart.ToString("yyyyMMdd");
            var msg = this.rtbMsg.Text;
            var wt = GetWorkType();
            var wo = this.GetWorkOut();
            var exp = Expressionable.Create<WorkUnit>().And(a => a.WorkDay == wd).ToExpression();
            var qu = this.Db.Queryable<WorkUnit>().Where(exp).ToList();
            if(qu.Any() && msg.IsNullOrEmpty()) {
                this.Db.Deleteable<WorkUnit>().Where(exp).ExecuteCommand();
                this.DataReset();
                return;
            }
            if(qu.Any() && msg.IsNotNullOrEmpty()) {
                this.Db.Updateable<WorkUnit>().Where(exp)
                    .SetColumns(a => a.WorkMessage == msg)
                    .SetColumns(a => a.WorkType == wt)
                    .SetColumns(a => a.Workout == wo)
                    .ExecuteCommand();
                this.DataReset();
                return;
            }
            if(!qu.Any() && msg.IsNotNullOrEmpty()) {
                this.Db.Insertable<WorkUnit>(new WorkUnit {
                    WorkDay = wd,
                    WorkMessage = msg,WorkType = wt,Workout = wo,
                }).ExecuteCommand();
                this.DataReset();
                return;
            }
        }

        private void btReset_Click(object sender,EventArgs e) {
            mcWorkDay_DateChanged(sender,new DateRangeEventArgs(this.mcWorkDay.SelectionStart,this.mcWorkDay.SelectionEnd));
            this.DataReset();
        }

        private void DataChanged() {
            this.btSave.Enabled = true;
            this.btReset.Enabled = true;
        }
        private void DataReset() {
            this.btSave.Enabled = false;
            this.btReset.Enabled = false;
        }

        /// <summary>
        /// 设置加班信息
        /// </summary>
        /// <param name="wo"></param>
        private void SetWorkOut(string wo) {
            if(wo.IsNullOrEmpty()) {
                return;
            }
            foreach(var c in this.C_WorkOut.Controls) {
                var ra = c as RadioButton;
                if(ra != null && ra.Text == wo) {
                    ra.Checked = true;
                    return;
                }
            }
        }
        /// <summary>
        /// 获取加班信息
        /// </summary>
        /// <returns></returns>
        private string GetWorkOut() {
            foreach(var c in this.C_WorkOut.Controls) {
                var ra = c as RadioButton;
                if(ra != null && ra.Checked) {
                    return ra.Text;
                }
            }
            return "";
        }

        private void rtbMsg_LinkClicked(object sender,LinkClickedEventArgs e) {
            try {
                System.Diagnostics.Process.Start(e.LinkText);
            }
            catch(Exception ex) {
                MessageBox.Show(ex.ToString());
            }
        }

        private void tsp_Copy_Click(object sender,EventArgs e) {
            Clipboard.SetText(this.rtbMsg.SelectedText);
        }

        private void tsb_palse_Click(object sender,EventArgs e) {
            this.rtbMsg.AppendText(Clipboard.GetText());
        }

        private void rtbMsg_TextChanged_1(object sender,EventArgs e) {

        }

        private void 删除ToolStripMenuItem_Click(object sender,EventArgs e) {
            var wd = this.mcWorkDay.SelectionStart.ToString("yyyyMMdd");
            var items = this.FileListView.SelectedItems;
            if(items.Count == 0) {
                MessageBox.Show("没有选择列表项！");
                return;
            }
            if(MessageBox.Show($"共选择了{items.Count}项，是否全部删除？","警告",MessageBoxButtons.OKCancel) == DialogResult.Cancel) {
                return;
            }
            foreach(ListViewItem i in items) {
                this.FileListView.Items.Remove(i);
                this.Db.Deleteable<WorkUnitPlusFile>()
                    .Where(a => a.WorkDay == wd && a.FileKey == i.SubItems[2].Text)
                    .ExecuteCommand();
            }
        }

        private void 导入ToolStripMenuItem_Click(object sender,EventArgs e) {
            var dlgFile = OpenFileDlg.ShowDialog();
            if(dlgFile == DialogResult.Cancel) {
                return;
            }
            if(OpenFileDlg.SafeFileNames.Length == 0) {
                return;
            }
            var wd = this.mcWorkDay.SelectionStart.ToString("yyyyMMdd");
            foreach(var f in OpenFileDlg.FileNames) {
                var lit1 = new ListViewItem(Path.GetFileName(f));
                lit1.SubItems.Add(new ListViewItem.ListViewSubItem { Text = "导入中" });
                lit1.SubItems.Add(new ListViewItem.ListViewSubItem { Text = "" });
                this.FileListView.Items.Add(lit1);

                var fc = PlusFile.LoadFile(f);
                var fk = PlusFile.GetMd5(fc);
                lit1.SubItems[2].Text = fk;
                var pfu = new PlusFileUnit { FileContent = fc,FileId = fk };
                var qu = this.Db.Queryable<PlusFileUnit>().Where(a => a.FileId == fk);
                if(!qu.Any()) {
                    this.Db.Insertable(pfu).ExecuteCommand();
                }
                this.Db.Insertable(new WorkUnitPlusFile {
                    FileKey = fk,
                    WorkDay = wd,
                    FileName = Path.GetFileName(f),
                }).ExecuteCommand();
            }
        }

        private void 下载ToolStripMenuItem_Click(object sender,EventArgs e) {
            var items = this.FileListView.SelectedItems;
            if(items.Count == 0) {
                MessageBox.Show("没有选择文件");
                return;
            }
            foreach(ListViewItem i in items) {
                var sdlg = new SaveFileDialog {
                    Title = "保存附件",
                    FileName = Path.GetFileName(i.Text),
                };
                var dresult = sdlg.ShowDialog();
                if(dresult != DialogResult.OK) {
                    return;
                }
                var filePath = sdlg.FileName;
                var fileKey = i.SubItems[2].Text;
                var qu = this.Db.Queryable<PlusFileUnit>().Where(a => a.FileId == fileKey).ToList();
                if(qu.Any()) {
                    var fir = qu.First();
                    if(fir.FileContent != null) {
                        PlusFile.SaveFile(filePath,fir.FileContent);
                    }
                }
            }
        }

        private void bt_m_ys_Click(object sender,EventArgs e) {
            var r = this.Db.DbVacuum();
            MessageBox.Show(r);
        }

        private void bt_m_Clean_Click(object sender,EventArgs e) {
            var r = this.Db.DbClean();
            MessageBox.Show(r);

        }

        private void bt_m_Backup_Click(object sender,EventArgs e) {
            var sdlg = new SaveFileDialog {
                Title = "备份数据库",
                FileName = Path.GetFileName($"workdb.{DateTime.Now.ToString("yyyyMMdd")}.bak"),
            };
            var dresult = sdlg.ShowDialog();
            if(dresult != DialogResult.OK) {
                return;
            }
            var filePath = sdlg.FileName;
            File.Copy(this.Db.SqliteDbfile,filePath);
            MessageBox.Show("备份完成");
        }
    }
}
